#include "huc.h"

/*  scroll-area size */
#define SCR_W    17
#define SCR_H    14
/*  screen size */
#define BAT_W    32
#define BAT_H    16
/*  map size */
#define MAP_W    256
#define MAP_H    96
/*  map init */
#define INIT_X   112
#define INIT_Y   77
/*  nb tile */
#define NB_TILE  135

/*  vram allocation */
#define TILE_VRAM    0x1000
#define SPRITE_VRAM  0x7000

/*  palette allocation */
#define TILE_PAL     4
#define SPRITE_PAL  16

/*  map */
#incbin(game_map,  "overworld.fmp")

/*  tiles */
#inctile(tile_gfx, "overworld.pcx", 27, 5)
#incpal(tile_pal,  "overworld.pcx", 0)

/*  palette index for tiles (one index per tile) */
const char tile_pal_ref[NB_TILE] = {
        0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40,
        0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40,
        0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40,
        0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40,
        0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40
};

int mapinfo[8];

/*mapping funcs below*/
screen_move(LinkInfo, x, y)
       int *LinkInfo, x, y;
{
  int i, addX, addY, direction;

  addX=(LinkInfo[2]&8)/8-(LinkInfo[2]&4)/4;
  addY=(LinkInfo[2]&2)/2-(LinkInfo[2]&1)/1;


  if(addY==-1){direction=0;}
  if(addY== 1){direction=1;}
  if(addX== 1){direction=2;}
  if(addX==-1){direction=3;}


  if(x!=0)
  {
    for(i=0;i<(256>>1);i++)
    {
      LinkInfo[0] -= x<<1;
      mapinfo[0] += x<<1;

      if(i>=120)
      {LinkInfo[0] += x<<1;}

      ShowLink(LinkInfo, direction);
      move(x<<1, 0);

      satb_update();
      vsync(1);
    }
  }

  if(y!=0)
  {
    for(i=0;i<176>>1;i++)
    {
      LinkInfo[1] -= y<<1;
      mapinfo[1] += y<<1;

      if(i>=80)
      {
        if(y>0 || i>=84)
        {LinkInfo[1] += y<<1;}
      }

      ShowLink(LinkInfo, direction);
      move(0, y<<1);

      satb_update();
      vsync(1);
    }
  }
}

init_map(x, y)
     int x, y;
{
  disp_off();
  vsync();

  cls();

  set_map_data(game_map, MAP_W, MAP_H);
  set_tile_data(tile_gfx, NB_TILE, tile_pal_ref);
  load_tile(TILE_VRAM);
  load_palette(TILE_PAL, tile_pal, 1);
  load_map(0, 0, x, y, SCR_W, SCR_H);

  mapinfo[0] = (x >> 4);
  mapinfo[1] = (y >> 4);
  mapinfo[2] = 0;
  mapinfo[3] = 0;
  mapinfo[4] = 0;
  mapinfo[5] = 0;
  mapinfo[6] = x;
  mapinfo[7] = y;

  scroll(0, 0, 0, 48, 223, 0xC0);

  disp_on();
  vsync();
}


move(dx, dy)
int dx;
int dy;
{
  int  x, y;
  char h;

  if(dx)
  {
    mapinfo[2] += dx;

    if (mapinfo[2] < 0)
    {
      mapinfo[2] &= 0x0F;
      mapinfo[4] -= 1;
      mapinfo[6] -= 1;

      if (mapinfo[4]  < 0)
        mapinfo[4] += BAT_W;
      if (mapinfo[6]  < 0)
        mapinfo[6] += MAP_W;

      load_column(mapinfo[4], mapinfo[5], mapinfo[6], mapinfo[7]);
    }
    else if (mapinfo[2] > 15)
    {
      mapinfo[2] &= 0x0F;
      mapinfo[4] += 1;
      mapinfo[6] += 1;

      if (mapinfo[4] >= BAT_W)
        mapinfo[4] -= BAT_W;
      if (mapinfo[6] >= MAP_W)
        mapinfo[6] -= MAP_W;

      load_column(mapinfo[4] + 16, mapinfo[5], mapinfo[6] + 16, mapinfo[7]);
    }
  }

  if (dy)
  {
    mapinfo[3] += dy;

    if (mapinfo[3] < 0)
    {
      mapinfo[3] &= 0x0F;
      mapinfo[5] -= 1;
      mapinfo[7] -= 1;

      if (mapinfo[5]  < 0)
        mapinfo[5] += SCR_H;
      if (mapinfo[7]  < 0)
        mapinfo[7] += MAP_H;

      load_row(mapinfo[4], mapinfo[5], mapinfo[6], mapinfo[7]);
    }
    else if (mapinfo[3] > 15)
    {
      mapinfo[3] &= 0x0F;
      mapinfo[5] += 1;
      mapinfo[7] += 1;

      if (mapinfo[5] >= SCR_H)
        mapinfo[5] -= SCR_H;
      if (mapinfo[7] >= MAP_H)
        mapinfo[7] -= MAP_H;

      load_row(mapinfo[4], mapinfo[5] + 12, mapinfo[6], mapinfo[7] + 12);
    }
  }

  x = (mapinfo[4] << 4) + mapinfo[2];
  y = (mapinfo[5] << 4) + mapinfo[3];

  h = (SCR_H << 4) - y+48;

  if (y <= 32)
  {
    scroll(0, x, y,   48, 223, 0xC0);
    scroll(1, 0, 0, 255, 255, 0x00);/*make sure second map moves out of the way*/
  }
  else
  {
    scroll(0, x, y, 48, h - 1, 0xC0);
    scroll(1, x, 0, h,   223, 0xC0);
  }
}

load_column(x, y, mx, my)
char x;
char y;
int  mx;
int  my;
{
  char h;
  h = SCR_H - y;
        load_map(x, y, mx, my,     1, h);
  if(y){load_map(x, 0, mx, my + h, 1, SCR_H - h);}
}

load_row(x, y, mx, my)
char x;
char y;
int  mx;
int  my;
{
  char w;

  if(y >= SCR_H){y -= SCR_H;}

  if ((x + SCR_W) > BAT_W)
  {
    w = BAT_W - x;
    load_map(x, y, mx,     my, w,         1);
    load_map(0, y, mx + w, my, SCR_W - w, 1);
  }
  else{load_map(x, y, mx, my, SCR_W, 1);}
}
/*mapping funcs above*/


/*Links funcs below*/
Link(LinkInfo)
int *LinkInfo;
{
  int addX,addY,direction;

  /* IF the counter isn't equal to the first normal position(stab button was pushed)
     AND it's not the reset position
     THEN set the controller so it is only pressing stab(so Link doesn't move(durring stab only)
  */
  if(LinkInfo[5]!=LinkInfo[7]-1 && LinkInfo[5]!=LinkInfo[7]){LinkInfo[2]=16;}

  addX=(LinkInfo[2]&8)/8-(LinkInfo[2]&4)/4;
  addY=(LinkInfo[2]&2)/2-(LinkInfo[2]&1)/1;

  /* IF the direction pad is pushed
     THEN change the array that holds the last direction Link was
          headed(used to determine record last direction pushed
  */
  if(LinkInfo[2]&15){LinkInfo[3]=LinkInfo[2];}

/*Crappy code below*/
  if(addY==-1){direction=0;if(LinkInfo[1]<= 47){addY=0;}}
  if(addY== 1){direction=1;if(LinkInfo[1]>=217){addY=0;}}
  if(addX== 1){direction=2;if(LinkInfo[0]>=241){addX=0;}}
  if(addX==-1){direction=3;if(LinkInfo[0]<=  -1){addX=0;}}
/*Crappy code above*/

  if(LinkInfo[2]!=16)
  {
    LinkInfo[0]+=addX;
    LinkInfo[1]+=addY;
  }

  ShowLink(LinkInfo, direction);

  /* IF the direction pad is active
     THEN +IF the counter is at 0
           THEN reset the counter
          +count down one in the counter
  */
  if(LinkInfo[2]&15)/* counts for walking animation */
  {
    if(!LinkInfo[4]){LinkInfo[4]=LinkInfo[7];}
    LinkInfo[4]--;
  }

  /* IF the stab button is pressed
     AND counter is not at reset position
     THEN +IF the counter is at 0
           THEN reset the counter to one higher than the normal reset
               (so you can tell you finished one cycle)
          +count down one in the counter
     ELSE reset the counter
  */
  if((LinkInfo[2]&16)/16 && LinkInfo[5]!=LinkInfo[7])/* counts for cycling animations (ie: stabbing) */
  {
    if(!LinkInfo[5])
    {
      LinkInfo[5]=LinkInfo[7]+1;

      spr_set(1);
      spr_pal(0);
      spr_pri(1);
      spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|NO_FLIP);
      spr_x(LinkInfo[0]+((LinkInfo[3]&8)/8-(LinkInfo[3]&4)/4)*0x10);
      spr_y(LinkInfo[1]+((LinkInfo[3]&2)/2-(LinkInfo[3]&1)/1)*0x10);
      spr_pattern(0x46C0);
    }
    LinkInfo[5]--;
  }

  /* IF the counter has just been reset
     AND stab button isn't still pressed
          THEN return into the normal cycle set
  */
  if(LinkInfo[5]==LinkInfo[7] && !((LinkInfo[2]&16)/16)){LinkInfo[5]--;}
}

ShowLink(LinkInfo, direction)
    int *LinkInfo, direction;
{

  spr_set(0);
  spr_pal(0);
  spr_pri(1);
  spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|NO_FLIP);
  spr_x(LinkInfo[0]);
  spr_y(LinkInfo[1]);


  if(LinkInfo[2]==16 && LinkInfo[5]!=LinkInfo[7])
  {
    /*starts at 0x4200 (stabbing Link) and adds 0x40(one 16x16 spirte position) for each forth(minus the
      first) of the animation cycle and adds 0x100(four 16x16 sprite pos) for each of the directions*/
    spr_pattern(0x4200+0x40*(LinkInfo[5]/(LinkInfo[7]/4))+0x100*direction);

    spr_set(1);
    spr_pal(0);
    spr_pri(1);
    spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|NO_FLIP);
    spr_x(LinkInfo[0]+((LinkInfo[3]&8)/8-(LinkInfo[3]&4)/4)*0x10);
    spr_y(LinkInfo[1]+((LinkInfo[3]&2)/2-(LinkInfo[3]&1)/1)*0x10);

    /*starts at 0x4600 (sword part) and adds 0x40(one 16x16 spirte position) for each forth of
      the animation cycle and adds 0x100(four 16x16 sprite pos) for each of the directions*/
    spr_pattern(0x4600+0x40*(LinkInfo[5]/(LinkInfo[7]/4))+0x100*direction);
  }
  else
  {
    /*starts at 0x4000 (walking Link) and adds 0x40(one 16x16 spirte position) for each half(minus the
      first) of the animation cycle and adds 0x80(two 16x16 sprite pos) for each of the directions*/
    spr_pattern(0x4000+0x40*(LinkInfo[4]/(LinkInfo[7]/2))+0x80*direction);
  }
}
/*Links funcs above*/

joystick(joy_num)
int joy_num;
{
  int data;
  data=0;

  if(joy(joy_num) & JOY_DOWN){data+=2;}
  else{if(joy(joy_num) & JOY_UP){data+=1;}}

  if(!data)
  {
    if(joy(joy_num) & JOY_LEFT){data+=4;}
    else{if(joy(joy_num) & JOY_RGHT){data+=8;}}
  }

  if(joy(joy_num) & JOY_A){data+=16;}
  if(joy(joy_num) & JOY_B){data+=32;}

  if(joy(joy_num) & JOY_STRT){data+=64;}
  if(joy(joy_num) & JOY_SLCT){data+=128;}

  return(data);
}

main()
{
/*
  LinkInfo Contents:

  0 = x position
  1 = y position
  2 = joystick data
  3 = old joystick data
  4 = a counter for walking animations

  5 = a counter for stabing or any other animation
      that resets if it isn't completed

  6 = holds the last position for reseting/cycling
      animations (so you can determine when a cycle is completed)

  7 = the maximum value for the counter (adjusting
      will change animation speed)
*/
  int LinkInfo[9];

  LinkInfo[0]=128;
  LinkInfo[1]=128;
  LinkInfo[2]=0;
  LinkInfo[3]=0;
  LinkInfo[4]=27;
  LinkInfo[5]=27;
  LinkInfo[6]=27;
  LinkInfo[7]=28;

  load_sprites(0x4000, Link_sprs, 40);
  set_sprpal(0, Link_pal);


  init_satb();

  init_map(INIT_X, INIT_Y);

  for (;;)
  {
    LinkInfo[2]=joystick(0);
    Link(LinkInfo);

    if(LinkInfo[1]<=47)
    {screen_move(LinkInfo, 0, -1);}

    if(LinkInfo[1]>=217)
    {screen_move(LinkInfo, 0, 1);}

    if(LinkInfo[0]<=-1)
    {screen_move(LinkInfo, -1, 0);}

    if(LinkInfo[0]>=241)
    {screen_move(LinkInfo, 1, 0);}


    satb_update();
    vsync(1);
  }

  return;
}

#incspr(Link_sprs, "LinkN.pcx", 8, 5);
#incpal(Link_pal, "LinkN.pcx");
